<?php

namespace Drupal\ohdear_integration\Plugin\monitoring\SensorPlugin;

use Drupal\Core\Form\FormStateInterface;
use Drupal\system\SystemManager;
use Drupal\monitoring\Entity\SensorConfig;
use Drupal\monitoring\Result\SensorResultInterface;
use Drupal\monitoring\SensorPlugin\ExtendedInfoSensorPluginInterface;
use Drupal\monitoring\SensorPlugin\SensorPluginBase;

/**
 * Watchdog aggregator which handles replacement of variables in the message.
 *
 * @SensorPlugin(
 *   id = "requirements_recap",
 *   label = @Translation("Requirements Recap"),
 *   description = @Translation("Aggregator of hook requirements messages."),
 *   addable = TRUE
 * )
 */
class HookRequirementsSensorPlugin extends SensorPluginBase implements ExtendedInfoSensorPluginInterface {

  /**
   * Sensor manager service.
   *
   * @var \Drupal\system\SystemManager
   */
  protected $systemManager;

  /**
   * {@inheritdoc}
   */
  public function __construct(SensorConfig $sensor_config, $plugin_id, $plugin_definition) {
    $this->systemManager = $this->getService('system.manager');
    parent::__construct($sensor_config, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public function runSensor(SensorResultInterface $sensor_result) {
    $severity = (int) $this->sensorConfig->getSetting('severity');
    $disabled_requirements = $this->sensorConfig->getSetting('disabled_requirements');
    // Configuration keys can't have dots in them and are replaced with `__`.
    array_walk($disabled_requirements, function (&$val) {
      $val = str_replace('.', '__', $val);
    });
    $requirements = array_diff_key($this->systemManager->listRequirements(), array_flip($disabled_requirements));
    $requirements = array_filter($requirements, function ($requirement) use ($severity) {
      return isset($requirement['severity']) && $requirement['severity'] === $severity;
    });
    $sensor_result->requirements = $requirements;
    $sensor_result->setValue(count($requirements));
    if (count($requirements)) {
      foreach ($requirements as $requirement) {
        $value = isset($requirement['value']) ? html_entity_decode((string) $requirement['value']) : '';
        $sensor_result->addStatusMessage(' @title@value', [
          '@title' => $requirement['title'] ? "{$requirement['title']}" : '',
          '@value' => $value ? ": {$value}" : '',
        ]);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function resultVerbose(SensorResultInterface $result) {
    if (!$requirements = $result->requirements) {
      return [];
    }
    $counter = 0;
    $output['table'] = [
      '#type' => 'table',
      '#header' => [
        'label' => [
          'data' => $this->t('Label'),
        ],
        'value' => [
          'data' => $this->t('Value'),
        ],
        'description' => [
          'data' => $this->t('Description'),
        ],
      ],
    ];
    $renderer = $this->getService('renderer');
    foreach ($requirements as $requirement) {
      $title = (string) $requirement['title'];
      $value = isset($requirement['value']) ? (string) $requirement['value'] : '';
      $description = $requirement['description'] ?? '';
      $description = is_array($description) ? $description : ['#markup' => (string) $description];
      $output['table'][$counter]['label'] = [
        '#type' => 'item',
        '#plain_text' => $title,
      ];
      $output['table'][$counter]['value'] = [
        '#type' => 'item',
        '#plain_text' => html_entity_decode($value),
      ];
      $output['table'][$counter]['description'] = [
        '#type' => 'item',
        '#markup' => html_entity_decode((string) $renderer->renderPlain($description)),
      ];
      $counter++;
    }
    return $output;
  }

  /**
   * {@inheritdoc}
   */
  public function getDefaultConfiguration() {
    $default_config = parent::getDefaultConfiguration();
    $default_config['settings']['severity'] = SystemManager::REQUIREMENT_ERROR;
    $default_config['category'] = $this->t('Requirements');
    $default_config['value_label'] = $this->t('Errors in status report');
    $default_config['description'] = $this->t('Number of errors in status report checks.');
    $default_config['value_type'] = 'number';
    $default_config['caching_time'] = 3600;
    $default_config['thresholds'] = [
      'type' => 'exceeds',
      'critical' => 1,
    ];
    $default_config['settings']['requirements'] = [];
    return $default_config;
  }

  /**
   * Adds UI for variables config object and key.
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);
    // Add weight to display config key before expected value.
    $form['severity'] = [
      '#type' => 'select',
      '#default_value' => $this->sensorConfig->getSetting('severity') ?? SystemManager::REQUIREMENT_ERROR,
      '#options' => $this->getSeverityOptions(),
      '#title' => $this->t('Requirements Severity'),
      '#description' => $this->t('Requirements of selected severity are checked in this sensor.'),
      '#required' => TRUE,
      '#weight' => -1,
    ];
    $all_requirements = $this->getAllRequirements();
    $form['disabled_requirements'] = [
      '#type' => 'checkboxes',
      '#default_value' => $this->sensorConfig->getSetting('requirements') ?? array_combine(array_keys($all_requirements), array_keys($all_requirements)),
      '#options' => $all_requirements,
      '#title' => $this->t('Disable requirements for sensor'),
      '#description' => $this->t('Select requirements that should not be a part of this sensor.'),
      '#weight' => 0,
    ];
    return $form;
  }

  /**
   * List of all requirements used of form options.
   *
   * @return array
   *   All requirements.
   */
  protected function getAllRequirements() {
    $requirements = $this->systemManager->listRequirements();
    $all_requirements = [];
    foreach ($requirements as $id => $requirement) {
      if (strpos($id, '.') !== FALSE) {
        $id = str_replace('.', '__', $id);
      }
      $all_requirements[$id] = is_string($requirement['title']) ? $requirement['title'] : $requirement['title']->__toString();
    }
    return $all_requirements;
  }

  /**
   * Get options for requirements severity.
   *
   * @return array
   */
  protected function getSeverityOptions() {
    return [
      SystemManager::REQUIREMENT_ERROR => $this->t('Error'),
      SystemManager::REQUIREMENT_WARNING => $this->t('Warning'),
      SystemManager::REQUIREMENT_OK => $this->t('Ok'),
    ];
  }

}
